#!/usr/bin/env perl

## --> Site Elevation Retriever, elev.pl
## --> Green Bay Professional Packet Radio, www.gbppr.org

## This file Copyright 2001 <contact@gbppr.org> under the GPL.
## NO WARRANTY.  Please send bug reports / patches.


# This script uses *FREE* elevation data provided by NOAA.  The data is taken
# at 30-second intervals.  It's not extremely accurate, but good enough for
# plotting radio paths.

if ($#ARGV != 0) {
  print "Usage: $0 /path/to/NOAA/30-second/elevation/data\n";
  exit 1;
}

# Global variables
#
$path = $ARGV[0];
$ofile = ".out";
$ubfile = ".ub";
$red = "\033[31m\033[1m";
$cyan = "\033[36m\033[1m";
$bld = "\033[37m\033[1m";
$mag = "\033[35m\033[1m";
$reset = "\033[0m";
$ver = "v1.0";
#
# End global variables

sub System {
if ((0xffff & system $args) != 0 ) {
  print STDERR "error: $!\n";
  exit 1;
 }
}

print "\33[H\33[J", $cyan, "#" x 80, $reset;
print $bld, "\n\n United States Elevation Retriever, $ver\n".
      " Green Bay Professional Packet Radio, www.gbppr.org\n\n", $reset;
      print $cyan, "#" x 80, $reset, "\n\n\n";

print "Site latitude data (deg/min/sec)\n".
      "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n";

print " Enter the ", $bld, "degrees", $reset, " (north) : ";
chomp($D1 = <STDIN>);
$D1 =~ tr/0-9//csd;

if ($D1 > 89 || $D1 < 0) {
  print "Degrees must be between 0-89\n";
  exit 1;
}

print "  Enter the ", $bld, "minutes", $reset, " (0-59) : ";
chomp($M1 = <STDIN>);
$M1 =~ tr/0-9//csd;

if ($M1 > 59 || $M1 < 0) {
  print "Minutes must be between 0-59\n";
  exit 1;
}

print "  Enter the ", $bld, "seconds", $reset, " (0-59) : ";
chomp($S1 = <STDIN>);
$S1 =~ tr/0-9//csd;

if ($S1 > 59 || $S1 < 0) {
  print "Seconds must be between 0-59\n";
  exit 1;
}

print "\n\nSite longitude data (deg/min/sec)\n".
      "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n";

print " Enter the ", $bld, "degrees", $reset, " (west) : ";
chomp($D2 = <STDIN>);
$D2 =~ tr/0-9//csd;

if ($D2 > 179 || $D2 < 0) {
  print "Degrees must be between 0-179\n";
  exit 1;
}

print " Enter the ", $bld, "minutes", $reset, " (0-59) : ";
chomp($M2 = <STDIN>);
$M2 =~ tr/0-9//csd;

if ($M2 > 59) {
  print "\nMinutes must be between 0-59\n";
  exit 1;
}

print " Enter the ", $bld, "seconds", $reset, " (0-59) : ";
chomp($S2 = <STDIN>);
$S2 =~ tr/0-9//csd;

if ($S2 > 59) {
  print "\nSeconds must be between 0-59\n";
  exit 1;
}

if ($S1 <= 15) {
  $S1 = 0;
}
elsif ($S1 >= 16 && $S1 <= 45) {
  $S1 = 30;
}
elsif ($S1 > 45 && $S1 <= 59) {
  if ($M1 == 59) {
    $S1 = 30;
  }
  else {
    $S1 = 0;
    $M1 = $M1 + 1;
  }
}
else {
  $S1 = 0;
}

if ($S2 <= 15) {
  $S2 = 0; 
} 
elsif ($S2 >= 16 && $S2 <= 45) {
  $S2 = 30;
} 
elsif ($S2 > 45 && $S2 <= 59) {
  if ($M2 == 59) { 
    $S2 = 30;
  } 
  else {
    $S2 = 0;
    $M2 = $M2 + 1;
 } 
} 
else {
  $S2 = 0;
} 

$y = (((($M1 * 60) + $S1) / 30) * 120) + 1;
$x = (($M2 * 60) + $S2) / 30;
$z = int($y) + int($x);

$file = $path . "/" ."Y" . $D1 . "X" . $D2 . ".30s";

print "\n-> Reading $file\n";

open F, $file || die "Can't open $file: $!\n";
while (<F>) {
  $lines++;
}
close F;

print "-> Unblocking $file\n";

if ($lines == 1) {
  $command = "dd if=$file of=$ubfile cbs=5 conv=unblock >/dev/null 2>&1";
  &System($args = "$command");
}
else {
  print "$file is not in the correct format or is missing.\n";
  exit 1;
}

open UB, $ubfile || die "Can't open $ubfile: $!\n";
open O, "> $ofile" || die "Can't open $ofile: $!\n";
while (<UB>) {
  chomp; 
  $i++;
  last if ($i == 14401);
  tr/0-9//csd;
  print O "$_\n";
}
close O;
close UB;

open F, $ofile || die "Can't open $ofile: $!\n";
  @array = <F>;
close F;

unlink $ubfile, $ofile;

$record = splice (@array, ($z - 1), 1);
chomp $record;
$meters = sprintf "%.f", $record * 0.3048;

print $mag, "\nNOAA 30-second data coordinates\n\n", $reset;
print "Lat : ", $red, "$D1-$M1-$S1", $reset, "\tLon : ", $red, "$D2-$M2-$S2\n", $reset;
print "Elevation : ", $red, $record, $reset, " feet ";
print "(", $red, $meters, $reset, " meters)\n\n";
